# -*- coding: UTF-8 -*-


from typing import Any, Iterator, Dict


class Response:
    @property
    def content(self) -> bytes: ...
    @property
    def cookies(self) -> Dict[str, str]: ...
    @property
    def headers(self) -> Dict[str, str]: ...
    @property
    def status_code(self) -> int: ...
    @property
    def url(self) -> str: ...
    @property
    def encoding(self) -> str: ...
    @property
    def text(self) -> str: ...
    def json(self) -> Any: ...
    def stream(self) -> Iterator[bytes]: ...
    @property
    def text_markdown(self) -> str: ...
    @property
    def text_plain(self) -> str: ...
    @property
    def text_rich(self) -> str: ...
